@@ -1,6 +1,6 @@
 #!/usr/bin/env ruby
 #
-#   Copyright 2006,2007,2008 Martin Pärtel <martin.partel@gmail.com>
+#   Copyright 2006,2007,2008,2009,2010 Martin Pärtel <martin.partel@gmail.com>
 #
 #   This file is part of bindfs.
 #
@@ -31,7 +31,9 @@ TESTDIR_NAME = 'tmp_test_bindfs'
 $only_these_tests = nil
 
 # Prepares a test environment with a mounted directory
-def testenv(testcase_title, bindfs_args, &block)
+def testenv(bindfs_args, &block)
+
+    testcase_title = bindfs_args
 
     return unless $only_these_tests == nil or $only_these_tests.member? testcase_title
 
@@ -117,6 +119,16 @@ def testenv(testcase_title, bindfs_args, &block)
     end
 end
 
+# Like testenv but skips the test if not running as root
+def root_testenv(bindfs_args, &block)
+    if Process.uid == 0
+        testenv(bindfs_args, &block)
+    else
+        puts "--- #{bindfs_args} ---"
+        puts "[  #{bindfs_args}  ]"
+        puts "SKIP (requires root)"
+    end
+end
 
 def assert
     raise Exception.new('test failed') unless yield
